"""A python package to help expedite insight generate from publicly available higher education datasets. """
__version__ = '0.102'

from pypeds.ipeds import *
from pypeds.datasets import *
from pypeds.views import *



